// the configured options and settings for LibKEA
#define LIBKEA_VERSION_MAJOR 1
#define LIBKEA_VERSION_MINOR 4
#define LIBKEA_VERSION_PATCH 13

#define LIBKEA_VERSION "1.4.13"
#define LIBKEA_PACKAGE_VERSION "1.4.13"
#define LIBKEA_PACKAGE_STRING "LibKEA 1.4.13"
#define LIBKEA_PACKAGE "LibKEA"
#define LIBKEA_PACKAGE_BUGREPORT "petebunting@mac.com"
#define LIBKEA_COPYRIGHT_YEAR "2013"
